@@ -18,13 +18,13 @@ from runr import Test_counts, Plot_counts, Heatmap, Compare_tests, Norm_from_cou
 from trivia import Test, As_fasta, As_gff, Sample, Stats
 from shred import Shred
 from igv import Make_genome, IGV_plots, As_userplots, Run_igv
-from workflows import Analyse_sample
+from workflows import Analyse_sample, Analyse_samples
 from variant import Freebayes, Vcf_filter, Snpeff, Vcf_nway, Vcf_patch, Test_variant_call, Power_variant_call
 
 from legion import *
 
 
-VERSION='0.90'
+VERSION='0.91'
 
 BOLD = '\x1b[1m'
 END = '\x1b[m'
@@ -176,7 +176,15 @@ samcount, for analysis with BioConductor packages.
 %(BOLD)sPipeline tools:%(END)s
     
     analyse-sample:
-                  - clip, align, and call consensus on a set of reads
+                  - Clip, align, and call consensus on a set of reads.
+                  
+    analyse-variants:
+                  - Produce a VCF file listing SNPs and other variants in
+                    a set of samples.
+
+    analyse-samples:
+                  - Run "analyse-sample:" on a set of different samples,
+                    then run "analyse-variants:".
 
 If a pipeline tools is run again, it restarts only from the point affected 
 by the changed parameters. The following global flags control pipeline tool 
@@ -190,7 +198,40 @@ behaviour:
   (GFF is not yet supported by all tools).
 - nesoni is able to read files compressed with gzip or bzip2.
 
-""" % { 'BOLD' : '\x1b[1m', 'END' : '\x1b[m', 'VERSION' : VERSION, 'MAKE' : Make().describe('', show_help=True) }
+
+%(BOLD)sSelections and sorts:%(END)s
+Working directories can be given a set of tags using "tag:". They also 
+implicitly have a tag for the name of the directory, and a tag "all".
+
+A %(BOLD)sselection expression%(END)s is a logical expression used to select a subset 
+of working directories. It may consist of (grouped by precedence):
+
+  tag        - Working directories with tag
+
+  [exp]      - exp
+
+  -exp       - not exp
+
+  exp1:exp2  - exp1 and exp2
+  exp1/exp2  - exp1 or exp2
+  exp1^expr2 - exp1 xor exp2
+
+Example:
+
+  [strain1:time1]/[-strain1:-time1]   
+             - Samples either from strain1 at time1, 
+               or not from strain1 and not from time1.
+               Equivalently: strain1^-time1
+
+A %(BOLD)ssort expression%(END)s is a comma separated list of selection expressions,
+used to sort a list of working directories.
+
+Example:
+
+  strain1,strain2,time1,time2,time3,replicate1,replicate2
+             - Sort, grouping by strain, then by time, then by replicate
+
+""" % { 'BOLD' : '\x1b[1m', 'END' : '\x1b[m', 'VERSION' : VERSION, 'MAKE' : Make().describe('', show_help=True, escape_newlines=False) }
 
 def get_actions():
     for item in globals().values():
