@@ -11,7 +11,7 @@ Metainformation about tools will hopefully allow
 """
 
 
-import sys, os, pickle, traceback, textwrap, re, copy, functools, types, datetime
+import sys, os, pickle, traceback, textwrap, re, copy, functools, types, datetime, pipes
 
 from nesoni import workspace
 
@@ -32,6 +32,11 @@ def filesystem_friendly_name(name):
 def colored(color, text):
     return '\x1b[%dm%s\x1b[m' % (color, text)
 
+def color_as_flag(text): return colored(1, text)
+def color_as_template(text): return colored(32, text)
+def color_as_value(text): return colored(35, text)
+def color_as_comment(text): return colored(36, text)
+
 def strip_color(text):
     return re.sub(r'\x1b\[\d*m','',text)
 
@@ -121,6 +126,9 @@ class Parameter(object):
     
     def describe(self, value):
         return str(value)
+    
+    def describe_quoted(self, value):
+        return pipes.quote(self.describe(value))
 
     def __call__(self, item):
         # Put the parameter after any parameters from base classes
@@ -147,7 +155,8 @@ class Parameter(object):
     def describe_shell(self, value, verbose=True):
         if value is None and not verbose: 
             return ''
-        return colored(1, self.shell_name()) + ' ' + colored(35 if value is not None else 34, self.describe(value))
+        return color_as_flag(self.shell_name()) + ' ' + \
+               (color_as_value if value is not None else color_as_template)(self.describe_quoted(value))
 
 
 class Hidden(Parameter):
@@ -159,7 +168,7 @@ class Positional(Parameter):
     sort_order = 1
 
     def shell_name(self):
-        return '%s' % self.name.replace('_','-').lower()
+        return '<%s>' % self.name.replace('_','-').lower()
     
     def parse(self, obj, string):
         expect_no_further_flags([string])
@@ -167,9 +176,9 @@ class Positional(Parameter):
     
     def describe_shell(self, value, verbose=True):
         if value is None:
-            return colored(34, self.shell_name()) if verbose else ''
+            return color_as_template(self.shell_name()) if verbose else ''
         else:
-            return colored(35, self.describe(value))
+            return color_as_value(self.describe(value))
 
 
 class Flag(Parameter):
@@ -233,8 +242,14 @@ class Section(Parameter):
     def shell_name(self):
         return self.name.replace('_','-').rstrip('-').lower()+':'
 
+    def describe_each(self, value):
+        return [ str(item) for item in value ]
+
     def describe(self, value):
-        return ' '.join(str(item) for item in value)
+        return ' '.join(self.describe_each(value))
+    
+    def describe_quoted(self, value):
+        return ' '.join([ pipes.quote(item) for item in self.describe_each(value) ])
 
     def describe_shell(self, value, verbose=True):
         if not verbose and not value: return ''
@@ -247,14 +262,24 @@ class Grouped_section(Section):
             expect_no_further_flags(args)
         return self.get(obj) + [ args ]
 
-    def describe(self, value):
-        return ' '.join(self.shell_name() + ' ' + ' '.join(item) for item in value)
-        
+    def describe_each(self, value):
+        return [ self.shell_name() + ' ' + ' '.join(item) for item in value ]
+    
+    def describe_quoted(self, value):
+        return ' '.join([ 
+            self.shell_name() + 
+            ' ' + 
+            ' '.join([ pipes.quote(item2) for item2 in item]) 
+            for item in value
+        ])
+    
     def describe_shell(self, value, verbose=True):
         if verbose and not value:
-            return colored(1, self.shell_name())
+            return color_as_flag(self.shell_name()) + ' ' + color_as_template('...')
         return '\n'.join(
-            colored(1, self.shell_name()) + ' ' + colored(35, ' '.join(item))
+            color_as_flag(self.shell_name()) + 
+            ' ' + 
+            color_as_value(' '.join(pipes.quote(item2) for item2 in item))
             for item in value
         )
             
@@ -263,24 +288,24 @@ class Float_section(Section):
     def parse(self, obj, args):
         return self.get(obj) + [ float(item) for item in args ]
 
-    def describe(self, value):
-        return ' '.join( '%f' % item for item in value )        
+    def describe_each(self, value):
+        return [ '%f' % item for item in value ]
 
 
 class Main_section(Section):
     sort_order = 2
 
     def shell_name(self):
-        return self.name.replace('_','-').rstrip('-').lower()+' ...'
+        return '<' + self.name.replace('_','-').rstrip('-').lower()+' ...>'
 
     def describe_shell(self, value, verbose=True):
         if not value:
             if verbose:
-                return colored(34, self.shell_name())
+                return color_as_template(self.shell_name())
             else:
                 return ''
         else:
-            return colored(35, self.describe(value))
+            return color_as_value(self.describe(value))
 
 
 class Configurable_section(Section):
@@ -300,10 +325,9 @@ class Configurable_section(Section):
         )
         self.presets = presets
         
-        if self.presets:
-            self.help += '\n\nChoose from the following, then supply extra flags as needed:\n'
-            for item in presets:
-                self.help += ' ' + item[0] + ' - ' + item[2] + '\n'
+        self.help += '\n\nChoose from the following, then supply extra flags as needed:\n'
+        for item in presets:
+            self.help += ' ' + item[0] + ' - ' + item[2] + '\n'
         
 
     def parse(self, obj, args):
@@ -335,14 +359,41 @@ class Configurable_section(Section):
                     preset_guess = item
                     break
         
-        result = colored(34, self.shell_name()) 
+        result = color_as_flag(self.shell_name()) 
         if preset_guess:
-            result += ' ' + colored(35,preset_guess[0])
+            result += ' ' + color_as_value(preset_guess[0])
         if value is not None and (not verbose or not preset_guess or (value != preset_guess[1])):
-            result += value.describe(invocation='',show_help=False,escape_newlines=False)
+            result += value.describe(invocation='',show_help=False,escape_newlines=False,brief=True)
         return result
 
 
+class Grouped_configurable_section(Section):
+    def __init__(self, name, help='', affects_output=True, empty_is_ok=True, template_getter=None):
+        super(Grouped_configurable_section,self).__init__(
+            name=name,
+            help=help,
+            affects_output=affects_output,
+            allow_flags=True,
+            empty_is_ok=empty_is_ok
+        )
+        self.template_getter = template_getter
+
+    def parse(self, obj, args):
+        new = self.template_getter(obj)
+        new.parse( args )
+        return self.get(obj) + [ new ]
+
+    def describe_shell(self, value, verbose=True):
+        if verbose and not value:
+            return color_as_flag(self.shell_name()) + ' ' + color_as_template('...')
+        return '\n'.join(
+            color_as_flag(self.shell_name()) + 
+            item.describe(invocation='',show_help=False,escape_newlines=False,brief=True).rstrip('\n')
+            for item in value
+        )
+        
+
+
 def help(short, extra=''):
     full = short
     if extra: full += '\n\n' + extra
@@ -525,9 +576,9 @@ class Configurable(object):
             if c: return c
         return 0
     
-    def describe(self, invocation=None, show_help=False, escape_newlines=True):
+    def describe(self, invocation=None, show_help=False, escape_newlines=True, brief=False):
         if invocation is None:
-            invocation = self.shell_name() + ':'
+            invocation = self.shell_name() + ':'            
     
         desc = [ colored(1, invocation) ]
         
@@ -543,11 +594,13 @@ class Configurable(object):
         
         for i in order:
             parameter = self.parameters[i]
+            if brief and parameter.get(self) == parameter.get(type(self)()):
+                continue
             line = parameter.describe_shell(parameter.get(self), show_help)
             if line:
                 desc.append(wrap(line,67,'    ',suffix))
                 if show_help and parameter.help:
-                    desc.append(colored(30,wrap(parameter.help,65,'      # ')))
+                    desc.append(colored(36,wrap(parameter.help,65,'      # ')))
        
         return (colored(2,suffix)+'\n').join( desc ) + '\n'
     
@@ -726,9 +779,15 @@ def report_exception():
     exception = sys.exc_info()[1]
     
     brief = False
-    for item in (Error, EnvironmentError):
-        if isinstance(exception, item) and len(exception.args) > 0:
-            brief = True
+    
+    if isinstance(exception, EnvironmentError) and \
+       exception.strerror in [ 
+           'No such file or directory' 
+           ]:
+       brief = True
+    
+    if isinstance(exception, Error) and len(exception.args) > 0:
+        brief = True
     
     if not brief:
         write_colored_text(sys.stderr, 
