@@ -8,7 +8,7 @@ SAM-based reboot
 import sys, os, subprocess, itertools, array, datetime, socket
 
 
-from nesoni import grace, bio, io, consensus, legion
+from nesoni import grace, bio, io, consensus, legion, config
 
 FLAG_PAIRED = 1
 FLAG_PROPER = 2
@@ -273,7 +273,7 @@ def bam_iter_fragments(filename, status_text='Processing'):
 class Bam_writer(object):
     def __init__(self, filename, headers=''):
         self.writer = io.Pipe_writer(filename, ['samtools', 'view', '-S', '-b', '-'])
-        self.writer.write(headers) 
+        self.writer.write(headers)
 
     def write(self, al):
         line = '\t'.join([
@@ -298,8 +298,6 @@ class Bam_writer(object):
         self.writer.close()
 
 
-
-
 def sort_and_index(in_filename, out_prefix):
     io.execute([
         'samtools', 'sort', in_filename, out_prefix
@@ -310,4 +308,33 @@ def sort_and_index(in_filename, out_prefix):
     ])
 
 
+@config.help("""\
+Merge and sort one or more bam files using Picard.
+""")
+@config.Main_section('bams')
+@config.String_flag('sort', 'Sort order. Can be "queryname" or "coordinate".')
+class Bam_merge(config.Action_with_prefix):
+    bams = [ ]
+    sort = 'coordinate'
+    
+    def run(self):
+        assert self.sort in ('queryname', 'coordinate')
+        
+        jar = io.find_jar('MergeSamFiles.jar', 'MergeSamFiles is part of the Picard package.')
+        io.execute([
+            'java','-jar',jar,
+            'USE_THREADING=true',
+            'SORT_ORDER='+self.sort,
+            'OUTPUT='+self.prefix+'.bam'
+            ] + [ 'INPUT='+item for item in self.bams ])
+        
+        if sort == 'coordinate':
+            jar = io.find_jar('BuildBamIndex.jar', 'BuildBamIndex is part of the Picard package.')
+            io.execute([
+                'java','-jar',jar,
+                'INPUT='+self.prefix+'.bam'
+                ])
+
+
+
 
