@@ -37,7 +37,7 @@ Useful shrimp options:
 """)
 @config.Main_section('references', 
     'Reference sequence filenames, '
-    'or a directory created using "nesoni make-reference: --bowtie yes" (recommended).')
+    'or a directory created using "nesoni make-reference:" (recommended).')
 @config.Bool_flag('cs', 'Are reads in colorspace?')
 @config.Bool_flag('sam_unaligned', 'Pass --sam-unaligned to gmapper?')
 @config.Bool_flag('half_paired', 'Pass --half-paired to gmapper?')
@@ -257,9 +257,8 @@ class Shrimp(config.Action_with_output_dir):
             else:
                 reads_parameters = [ '-1', filenames[0], '-2', filenames[1] ]
             
-            default_options['--read-group'] = '%s-%d,%s' % (
+            default_options['--read-group'] = '%s,%s' % (
                 workspace.name.replace(',','_'),
-                i+1,
                 workspace.name.replace(',','_')
             )
             for flag in default_options:
